#include "RooGlobalFunc.h"
#include "RooRealVar.h"
#include "RooArgList.h"
#include "RooFormulaVar.h"
#include "RooDataSet.h"
#include "RooGaussian.h"
#include "RooChebychev.h"
#include "RooExponential.h"
#include "RooBifurGauss.h"
#include "RooAddModel.h"
#include "RooProdPdf.h"
#include "TCanvas.h"
#include "RooPlot.h"
#include "RooHist.h"
#include "RooCBShape.h"
#include "RooPolynomial.h"
#include "RooBinning.h"
#include "TH1.h"
#include "TH2.h"
#include "RooAddPdf.h"
#include "RooProdPdf.h"
#include "RooFitResult.h"
#include "RooGenericPdf.h"
#include "RooLandau.h"
#include "TChain.h"
#include "TCanvas.h"
#include "TAxis.h"
#include "RooPlot.h"
#include "RooCategory.h"
#include "RooSuperCategory.h"
#include "RooSimultaneous.h"
#include "RooNLLVar.h"
#include "TFile.h"
#include "RooStats/SPlot.h"
#include "RooAbsDataStore.h"
#include "RooFit.h"
#include "TPaveLabel.h"
#include "RooJohnson.h"
#include "RooPolyVar.h"
#include "myRooPolBG.cpp"
#include "myRooJohnsonSU.cpp"

using namespace RooFit;
using namespace std;
using namespace RooStats;

void splot_data_kk_new()  
{
	
  TFile *ft = new TFile("b2_data_kk_iriomote_bcs.root");

  TTree *tree = (TTree*)ft->Get("DstD0PiKK");
  Int_t nch =(int)tree->GetEntries();
  cout << "Number of Entries in data files = "<< nch << endl;

  Double_t Dz_M, DzPi_M;
  Double_t pisoft_p, pisoft_cosTheta,Dst_p, Dst_cosTheta;
  
  tree->SetBranchAddress("Dst_p",&Dst_p);
  tree->SetBranchAddress("Dst_cosTheta",&Dst_cosTheta);
  tree->SetBranchAddress("pisoft_p",&pisoft_p);
  tree->SetBranchAddress("pisoft_cosTheta",&pisoft_cosTheta);
  tree->SetBranchAddress("Dz_M",&Dz_M);
  tree->SetBranchAddress("DzPi_M",&DzPi_M);

  RooRealVar Dstp("Dst_p","", 1.5, 6.5, "");
  RooRealVar Dstcos("Dst_cosTheta","", -1, 1, "");
  RooRealVar pisoftp("pisoft_p","", 0.05, 0.6, "");
  RooRealVar pisoftcos("pisoft_cosTheta","", -1, 1, "");
  RooRealVar mKK("Dz_M","", 1.75, 2.05, "");
  RooRealVar mD0pi("DzPi_M","", 2.005, 2.025, "");

  auto variables = new RooArgSet();
  variables->add(Dstp);
  variables->add(Dstcos);
  variables->add(pisoftp);
  variables->add(pisoftcos);
  variables->add(mKK);
  variables->add(mD0pi);

  RooDataSet datat("datat","datat", tree, RooArgSet(*variables));
  // Construct formula to calculate (fake) weight for events
  //RooFormulaVar wFunc("w","event weight","0.889752 -0.110344*(Dst_cosTheta) + 0.961291*(pow(Dst_cosTheta,2))-0.220356*(pow(Dst_cosTheta,3))-0.0826763*(pow(Dst_cosTheta,4))-3.09148*(pow(Dst_cosTheta,5))+2.45831*(pow(Dst_cosTheta,6))",Dst_cosTheta) ;
  
  // RooPolyVar wFunc("w","event weight",Dstcos,RooArgList(RooConst(0.889752),RooConst(-0.110344),RooConst(0.961291),RooConst(-0.220356),RooConst(-0.0826763),RooConst(-3.09148),RooConst(2.45831)),0);// Add column with variable w to previously generated dataset
  RooFormulaVar wFunc("w","event weight","(Dstcos*Dstcos+10)",Dstcos) ;
  RooRealVar* w = (RooRealVar*) datat.addColumn(wFunc);

  // Instruct dataset wdata in interpret w as event weight rather than as observable
  // RooDataSet wdata(datat.GetName(),datat.GetTitle(),datat,datat.get(),0,*w);
  // RooDataSet wdata(datat.GetName(),datat.GetTitle(),tree, RooArgSet(*variables),w->GetName());
   RooDataSet wdata(datat.GetName(),datat.GetTitle(),datat,*datat.get(),0,w->GetName()) ;

  ////////////////////////// fitting part data///////////////////////////////
  /*-----------------------S I G N A L----------------------*/
  
  //d0pi
  RooRealVar f_J_sig("f_{Jsig}", "rel_frac_j",0.8937);
  RooRealVar mu_J_sig("#mu_{Jsig}","mean_johnson",2.0102529);
  RooRealVar sigma_J_sig("#sigma_{Jsig}", "sigma_johnson",0.0001675);
  RooRealVar gamma_J_sig("#gamma_{Jsig}", "gamma", -0.0329);
  RooRealVar delta_J_sig("#delta_{Jsig}", "delta", 1.033);
  myRooJohnsonSU johnson_sig("johnson_sig","johnson_sig", mD0pi, mu_J_sig, sigma_J_sig, delta_J_sig, gamma_J_sig);
  
  RooRealVar f_G1_sig("f_{G1sig}", "frac_gauss1",0.136);
  RooRealVar mu_G1_sig("#mu_{G1sig}", "mean_gauss1",2.010300);
  RooRealVar sigma_G1_sig("#sigma_{G1sig}", "sigma_gauss1",0.00385);
  RooRealVar sigma_G2_sig("#sigma_{G2sig", "sigma_gauss2",0.001224);
  RooGaussian gauss1_sig("gauss1_sig", "1st gaussian PDF", mD0pi, mu_G1_sig, sigma_G1_sig);
  RooGaussian gauss2_sig("gauss2_sig", "2nd gaussian PDF", mD0pi, mu_G1_sig, sigma_G2_sig);
  RooAddPdf doublegaussian_sig("doublegaussian_sig", "doublegaussian_sig", gauss1_sig, gauss2_sig, f_G1_sig);
  
  RooAddPdf sig_mD0pi("sig_mD0pi","sig_mD0pi", johnson_sig, doublegaussian_sig, f_J_sig);
  
  //kk
  RooRealVar f_J_sig_d0("f_{J_sig_d0}", "relative_frac_j",0.9624);
  RooRealVar mu_J_sig_d0("#mu_{J_sig_d0}","mean_johnson",1.864738);
  RooRealVar sigma_J_sig_d0("#sigma_{J_sig_d0}", "sigma_johnson", 0.00612);
  RooRealVar gamma_J_sig_d0("#gamma_{J_sig_d0}", "gamma",0.011);
  RooRealVar delta_J_sig_d0("#delta_{J_sig_d0}", "delta",1.689);
  myRooJohnsonSU johnson_sig_d0("johnson_sig_d0", "johnson_sig_d0", mKK, mu_J_sig_d0, sigma_J_sig_d0, delta_J_sig_d0, gamma_J_sig_d0 );
  
  RooRealVar f_G1_sig_d0("f_{G1}_sig_d0", "relative_frac_g",0.415);
  RooRealVar mu_G1_sig_d0("#mu_{G1}_sig_d0", "mean_gauss1",1.8506);
  RooRealVar sigma_G1_sig_d0("#sigma_{G1}_sig_d0", "sigma_gauss1",0.0616);
  RooRealVar sigma_G2_sig_d0("#sigma_{G2}_sig_d0", "sigma_gauss2",0.0225);
  RooGaussian gauss1_sig_d0("gauss1_sig_d0", "1st gaussian PDF", mKK, mu_G1_sig_d0, sigma_G1_sig_d0);
  RooGaussian gauss2_sig_d0("gauss2_sig_d0", "2nd gaussian PDF", mKK, mu_G1_sig_d0, sigma_G2_sig_d0);
  RooAddPdf doublegaussian_sig_d0("doublegaussian_sig_d0", "doublegaussian_sig_d0", gauss1_sig_d0, gauss2_sig_d0, f_G1_sig_d0);
  RooAddPdf sig_mKK("sig_mKK","sig_mKK", johnson_sig_d0, doublegaussian_sig_d0, f_J_sig_d0);
  
  //signal peak
  RooProdPdf signal_peak("signal_peak", "signal_peak", sig_mKK, sig_mD0pi);
  
  //random pion (common for all)
  RooRealVar threshold("threshold", "no data beyond this", 2.00441);
  RooRealVar alpha("#alpha", "alpha", -15.639);
  RooRealVar beta("#beta", "beta", 0.);
  myRooPolBG bkg_mD0pi("bkg_mD0pi","bkg_mD0pi", mD0pi, threshold, alpha, beta);
  
  //signal & random pion
  RooProdPdf signal_rnd("signal_rnd", "signal_rnd", sig_mKK, bkg_mD0pi);
  
  /*-----------------------M U L T I B O D Y-------------------------*/
  
  //d0pi
  RooRealVar delta_J_mult("#delta_{J}", "delta_mult",-1.2859);
  RooRealVar mu_J_mult("#mu_{J}_mult","mean_johnson",2.008198);
  RooRealVar sigma_J_mult("#sigma_{J}_mult", "sigma_johnson",0.00363);
  RooRealVar gamma_J_mult("#gamma_{J}_mult", "gamma", 1.);
  
  myRooJohnsonSU mult_mD0pi("mult_mD0pi","mult_mD0pi", mD0pi, mu_J_mult, sigma_J_mult, delta_J_mult, gamma_J_mult);
  
  //kk
  RooRealVar alpha1("a_{1}", "A1", -12.388);
  RooExponential mult_mKK("mult_mKK", "mult_mKK", mKK, alpha1);  

  RooProdPdf mult_peak("mult_peak", "mult_peak", mult_mKK, mult_mD0pi);
  
  //multibody & random pion
  RooProdPdf mult_rnd("mult_rnd","mult_rnd", mult_mKK, bkg_mD0pi);
  
  /*-----------------------K Pi-------------------------*/
  
  //kk
  RooRealVar f1_d0kpi("f_{G}d0kpi", "relative fraction gaussian",0.388);//0.385);
  RooRealVar mu_G_d0kpi("#mu_{G}d0kpi","mean_gaus",1.94123);//1.94120);                                                          
  RooRealVar sigma_G_d0kpi("#sigma_{G}d0kpi", "sigma_gaus",0.00718 );//0.00716);
  RooGaussian gauss_d0kpi("gauss_d0kpi", "gauss component", mKK, mu_G_d0kpi, sigma_G_d0kpi);
  
  RooRealVar gamma_J_d0kpi("#gamma_{J}d0kpi", "gamma", 1.288);//1.302);
  RooRealVar delta_J_d0kpi("#delta_{J}d0kpi", "delta", -0.3525);//-0.3516);
  RooRealVar mu_J_d0kpi("#mu_{J}_d0kpi", "mean",1.95406);//1.95397);
  RooRealVar sigma_J_d0kpi("#sigma_{J}_d0kpi", "sigma",0.00762);//0.00760);
  myRooJohnsonSU johnson_d0kpi("johnson_d0kpi", "Johnson PDF", mKK, mu_J_d0kpi, sigma_J_d0kpi, delta_J_d0kpi, gamma_J_d0kpi);
  
  RooAddPdf kpi_mKK("kpi_mKK","kpi_mKK",gauss_d0kpi,johnson_d0kpi,f1_d0kpi);
  
  //kpi peaking background
  RooProdPdf kpi_peak("kpi_peak", "kpi_peak", kpi_mKK, sig_mD0pi);//as the shape of m(d0pi) is same as signal
  
  //kpi & random pion
  RooProdPdf kpi_rnd("kpi_rnd", "kpi_rnd", kpi_mKK, bkg_mD0pi);

  /*----------------------- Ds+ -> K+ K- pi+ -------------------------*/
  //d0pi
  RooRealVar slope_ds("slope_ds", "Slope1 of Polynomial",0.797);
  RooChebychev ds_mD0pi("ds_mD0pi","Chebshev Polynomial", mD0pi,RooArgList(slope_ds));

  //kk
  RooRealVar mu_0("#mu_{0}", "",0.000214269);
  RooRealVar constant("constant", "",3.83333);
  RooFormulaVar mu_ds("#mu_{ds}", "@0+@1-@2",RooArgList(mu_0,constant, mD0pi));
  RooRealVar sigma_ds("#sigma_{ds}", "",0.00684);
  RooRealVar gamma_ds("#gamma_{ds}", "",-0.1118);
  RooRealVar delta_ds("#delta_{ds}", "",1.125);

  myRooJohnsonSU ds_mKK("ds_mKK","johnson",mKK, mu_ds,sigma_ds,delta_ds,gamma_ds);
 
  RooProdPdf ds_peak("ds_peak", "ds_peak", ds_mD0pi, Conditional(ds_mKK, mKK));  
  
  /*------------------------C O M B I N A T O R I A L----------------------------*/
  //Chebychev
  RooRealVar slope1("slope1", "Slope1 of Polynomial",-0.17);
  RooChebychev comb_mKK("chebpol","Chebshev Polynomial", mKK,RooArgList(slope1));
  RooProdPdf comb_bkg("comb_bkg", "comb_bkg", comb_mKK, bkg_mD0pi);
  
  /*------------------------Y I E L D S  F O R  A L L  C O M P O N E N T S----------------------------*/

  RooRealVar sig_peak_yield("sig_peak_yield","signal yield",148780,0,1e7);
  RooRealVar sig_randompi_yield("sig_randompi_yield","signal + random-pi yield",12840,0,1e7);
  
  RooRealVar mult_peak_yield("mult_peak_yield","multibody yield",69282,0,1e7);
  RooRealVar mult_randompi_yield("mult_randompi_yield","multibody + random-pi yield",0,1e5);
  
  RooRealVar kpi_peak_yield("kpi_peak_yield","kpi yield",104512,0,1e7);
  RooRealVar kpi_randompi_yield("kpi_randompi_yield","kpi + random-pi yield",9918,0,1e7);

  RooRealVar ds_peak_yield("ds_peak_yield","ds yield",2962,0,1e7);
  
  RooRealVar comb_yield("comb_yield","combinatorial yield",122105,0.,1e7);
    
  /*------------------------C O M P L E T E  M O D E L----------------------------*/
  RooAddPdf pdf("pdf","",RooArgList(signal_peak, signal_rnd, mult_peak, mult_rnd, kpi_peak, kpi_rnd, ds_peak, comb_bkg), RooArgList(sig_peak_yield,sig_randompi_yield,mult_peak_yield, mult_randompi_yield, kpi_peak_yield,kpi_randompi_yield,ds_peak_yield,comb_yield));


  std::cout << "\n\n------------------------------------------\nThe dataset before creating sWeights:\n";
  wdata.Print();

  TCanvas *ca1 = new TCanvas("ca1" ,"ca1", 800, 800);
  ca1->Divide(3, 2);
  	
  /*------------------------S P L O T----------------------------*/
  	
 // Now we use the SPlot class to add SWeights to our data set based on our model and our yield variables
  RooStats::SPlot* sDataX = new RooStats::SPlot("sData","An SPlot", wdata, &pdf, RooArgList(sig_peak_yield,sig_randompi_yield,mult_peak_yield, mult_randompi_yield, kpi_peak_yield,kpi_randompi_yield,ds_peak_yield,comb_yield));


  std::cout << "\n\n-------------------------------------------\nThe dataset after creating sWeights:\n";
  wdata.Print();
  
	// check that out weight have the desired property
	std::cout << "Check SWeights:" << std::endl;

  // Check that our weights have the desired properties
  std::cout << "\n\n------------------------------------------\n\nCheck SWeights:" << std::endl;
  cout <<"N_{sig}"        << sig_peak_yield.getVal()     << ": From s_weight N_{sig}"         << sDataX->GetYieldFromSWeight("signal_peak_yield")     <<endl;
  // cout <<"N_{peaking}"    << kspipi_yield.getVal()     << ": From s_weight N_{kspipi}"      << sDataX->GetYieldFromSWeight("kspipi_yield")     <<endl;
  // cout <<"N_{nonpeaking}" << nonpeaking_yield.getVal() << ": From s_weight N_{non_peaking}" << sDataX->GetYieldFromSWeight("nonpeaking_yield") <<endl;

  std::cout << std::endl;

  std::cout << "weighted data set: "<< std::endl;

  //create weighted data set
  RooDataSet* dataw_sig        = new RooDataSet(wdata.GetName(), wdata.GetTitle(), &wdata, *wdata.get(), 0, "sig_peak_yield_sw");
  // RooDataSet* dataw_kspipi     = new RooDataSet(datat.GetName(), datat.GetTitle(), &datat, *datat.get(), 0, "kspipi_yield_sw") ;
  // RooDataSet* dataw_nonpeaking = new RooDataSet(datat.GetName(), datat.GetTitle(), &datat, *datat.get(), 0, "nonpeaking_yield_sw") ;
    
  // Weighted histograms
  TH1F* hsig_dst_p      = new TH1F("hsig_dst_p"     , "", 100, 1.5, 6.5);
  TH1F* hsig_dst_cos    = new TH1F("hsig_dst_cos"   , "", 100, -1, 1);

  TH1F* hsig_pisoft_p   = new TH1F("hsig_pisoft_p"  , "", 100,  0.05, 0.6);
  TH1F* hsig_pisoft_cos = new TH1F("hsig_pisoft_cos", "", 100, -1, 1);

  dataw_sig->fillHistogram(hsig_dst_p,   RooArgList(Dstp));
  dataw_sig->fillHistogram(hsig_dst_cos, RooArgList(Dstcos));

  dataw_sig->fillHistogram(hsig_pisoft_p,   RooArgList(pisoftp));
  dataw_sig->fillHistogram(hsig_pisoft_cos, RooArgList(pisoftcos));

  cout << "Making splots of the signal and background" <<endl;

  //////////////////////// splot for signal //////////////////
  RooPlot* frame_sig_dstp = Dstp.frame(100) ;
  frame_sig_dstp->SetTitle("sPlot: Dst_p"); 
  // Since the data are weighted, we use SumW2 to compute the errors.
  dataw_sig->plotOn(frame_sig_dstp, RooFit::DataError(RooAbsData::SumW2));

  RooPlot* frame_sig_dstcos = Dstcos.frame(100) ;
  frame_sig_dstcos->SetTitle("sPlot: Dst_cos"); 
  dataw_sig->plotOn(frame_sig_dstcos, RooFit::DataError(RooAbsData::SumW2));

  RooPlot* frame_sig_pisoftp = pisoftp.frame(100) ;
  frame_sig_pisoftp->SetTitle("sPlot: pisoft_p"); 
  // Since the data are weighted, we use SumW2 to compute the errors.
  dataw_sig->plotOn(frame_sig_pisoftp, RooFit::DataError(RooAbsData::SumW2));

  RooPlot* frame_sig_pisoftcos = pisoftcos.frame(100) ;
  frame_sig_pisoftcos->SetTitle("sPlot: pisoft_cos"); 
  dataw_sig->plotOn(frame_sig_pisoftcos, RooFit::DataError(RooAbsData::SumW2));


  // plot PDFs onto the sPlots; if the sPlot correctly reweights the data then the signal / background only shapes should
  // provide an adequate description of the data.
  ca1->cd(1);
  frame_sig_dstp->SetXTitle("Momenta (D^{*+})");
  frame_sig_dstp->GetXaxis()->CenterTitle();
  frame_sig_dstp->GetXaxis()->SetTitleSize(0.05);
  frame_sig_dstp->Draw();

  ca1->cd(2);
  frame_sig_dstcos->SetXTitle("Cos #theta (D^{*+})");
  frame_sig_dstcos->GetXaxis()->CenterTitle();
  frame_sig_dstcos->GetXaxis()->SetTitleSize(0.05);
  frame_sig_dstcos->Draw();

  ca1->cd(3);
  frame_sig_pisoftp->SetXTitle("Momenta (#pi_{s})");
  frame_sig_pisoftp->GetXaxis()->CenterTitle();
  frame_sig_pisoftp->GetXaxis()->SetTitleSize(0.05);
  frame_sig_pisoftp->Draw();

  ca1->cd(4);
  frame_sig_pisoftcos->SetXTitle("Cos #theta (#pi_{s})");
  frame_sig_pisoftcos->GetXaxis()->CenterTitle();
  frame_sig_pisoftcos->GetXaxis()->SetTitleSize(0.05);
  frame_sig_pisoftcos->Draw();

    
  cout<< "Weight signal_Dst_p      (sPlot) = " << hsig_dst_p->Integral()      << endl; 
  cout<< "Weight signal_Dst_cos    (sPlot) = " << hsig_dst_cos->Integral()    << endl; 
  cout<< "Weight signal_pisoft_p   (sPlot) = " << hsig_pisoft_p->Integral()   << endl; 
  cout<< "Weight signal_pisoft_cos (sPlot) = " << hsig_pisoft_cos->Integral() << endl; 

  // Output file
  TFile *file = new TFile("sPlot_kk_data.root", "recreate");
  hsig_dst_p->Write(); 
  hsig_dst_cos->Write(); 

  hsig_pisoft_p->Write();
  hsig_pisoft_cos->Write();

  file->Close();


  
  
}












